@@ -147,7 +147,7 @@ def _prepare_from_authority(auth_name: str, auth_code: Union[str, int]):
 
 
 def _prepare_from_epsg(auth_code: Union[str, int]):
-    return _prepare_from_authority("epsg", auth_code)
+    return _prepare_from_authority("EPSG", auth_code)
 
 
 def _is_epsg_code(auth_code: Any) -> bool:
@@ -180,7 +180,7 @@ class CRS:
 
     """
 
-    def __init__(self, projparams: Any = None, **kwargs) -> None:
+    def __init__(self, projparams: Optional[Any] = None, **kwargs) -> None:
         """
         Initialize a CRS class instance with:
           - PROJ string
@@ -642,7 +642,7 @@ class CRS:
             CF-1.8 version of the projection.
 
         """
-        # pylint: disable=too-many-branches
+        # pylint: disable=too-many-branches,too-many-return-statements
         cf_dict: Dict[str, Any] = {"crs_wkt": self.to_wkt(wkt_version)}
 
         # handle bound CRS
@@ -693,23 +693,31 @@ class CRS:
         # handle geographic CRS
         if self.geodetic_crs:
             cf_dict["geographic_crs_name"] = self.geodetic_crs.name
+            if self.geodetic_crs.datum:
+                cf_dict["horizontal_datum_name"] = self.geodetic_crs.datum.name
 
         if self.is_geographic:
             if self.coordinate_operation:
+                if (
+                    self.coordinate_operation.method_name.lower()
+                    not in _INVERSE_GEOGRAPHIC_GRID_MAPPING_NAME_MAP
+                ):
+                    if errcheck:
+                        warnings.warn(
+                            "Unsupported coordinate operation: "
+                            f"{self.coordinate_operation.method_name}"
+                        )
+                    return {"crs_wkt": cf_dict["crs_wkt"]}
                 cf_dict.update(
                     _INVERSE_GEOGRAPHIC_GRID_MAPPING_NAME_MAP[
                         self.coordinate_operation.method_name.lower()
                     ](self.coordinate_operation)
                 )
-                if self.datum:
-                    cf_dict["horizontal_datum_name"] = self.datum.name
             else:
                 cf_dict["grid_mapping_name"] = "latitude_longitude"
             return cf_dict
 
         # handle projected CRS
-        if self.is_projected and self.datum:
-            cf_dict["horizontal_datum_name"] = self.datum.name
         coordinate_operation = None
         if not self.is_bound and self.is_projected:
             coordinate_operation = self.coordinate_operation
@@ -729,7 +737,7 @@ class CRS:
                 else:
                     warnings.warn("Coordinate operation not found.")
 
-            return {"crs_wkt": self.to_wkt(wkt_version)}
+            return {"crs_wkt": cf_dict["crs_wkt"]}
 
         cf_dict.update(
             _INVERSE_GRID_MAPPING_NAME_MAP[coordinate_operation_name](
@@ -741,9 +749,9 @@ class CRS:
     @staticmethod
     def from_cf(
         in_cf: dict,
-        ellipsoidal_cs: Any = None,
-        cartesian_cs: Any = None,
-        vertical_cs: Any = None,
+        ellipsoidal_cs: Optional[Any] = None,
+        cartesian_cs: Optional[Any] = None,
+        vertical_cs: Optional[Any] = None,
     ) -> "CRS":
         """
         .. versionadded:: 2.2.0
@@ -1216,12 +1224,9 @@ class CRS:
         """
         wkt = self._crs.to_wkt(version=version, pretty=pretty)
         if wkt is None:
-            warnings.warn(
+            raise CRSError(
                 f"CRS cannot be converted to a WKT string of a '{version}' version. "
-                "Select a different version of a WKT string or edit your CRS. Future"
-                "versions of pyproj will raise a CRSError in this case.",
-                FutureWarning,
-                stacklevel=2,
+                "Select a different version of a WKT string or edit your CRS."
             )
         return wkt
 
@@ -1244,12 +1249,7 @@ class CRS:
         """
         proj_json = self._crs.to_json(pretty=pretty, indentation=indentation)
         if proj_json is None:
-            warnings.warn(
-                "CRS cannot be converted to a PROJ JSON string. "
-                "Future versions of pyproj will raise a CRSError in this case.",
-                FutureWarning,
-                stacklevel=2,
-            )
+            raise CRSError("CRS cannot be converted to a PROJ JSON string.")
         return proj_json
 
     def to_json_dict(self) -> dict:
@@ -1285,12 +1285,7 @@ class CRS:
         """
         proj = self._crs.to_proj4(version=version)
         if proj is None:
-            warnings.warn(
-                "CRS cannot be converted to a PROJ string. "
-                "Future versions of pyproj will raise a CRSError in this case.",
-                FutureWarning,
-                stacklevel=2,
-            )
+            raise CRSError("CRS cannot be converted to a PROJ string.")
         return proj
 
     def to_epsg(self, min_confidence: int = 70) -> Optional[int]:
@@ -1301,7 +1296,7 @@ class CRS:
         Example:
 
         >>> from pyproj import CRS
-        >>> ccs = CRS("epsg:4328")
+        >>> ccs = CRS("EPSG:4328")
         >>> ccs.to_epsg()
         4328
 
@@ -1340,7 +1335,7 @@ class CRS:
         Example:
 
         >>> from pyproj import CRS
-        >>> ccs = CRS("epsg:4328")
+        >>> ccs = CRS("EPSG:4328")
         >>> ccs.to_authority()
         ('EPSG', '4328')
 
@@ -1383,7 +1378,7 @@ class CRS:
         Example:
 
         >>> from pyproj import CRS
-        >>> ccs = CRS("epsg:4328")
+        >>> ccs = CRS("EPSG:4328")
         >>> ccs.list_authority()
         [AuthorityMatchInfo(auth_name='EPSG', code='4326', confidence=100)]
 
@@ -1443,7 +1438,7 @@ class CRS:
         """
         This checks if the CRS is geographic.
         It will check if it has a geographic CRS
-        in the sub CRS if it is a compount CRS and will check if
+        in the sub CRS if it is a compound CRS and will check if
         the source CRS is geographic if it is a bound CRS.
 
         Returns
@@ -1458,7 +1453,7 @@ class CRS:
         """
         This checks if the CRS is projected.
         It will check if it has a projected CRS
-        in the sub CRS if it is a compount CRS and will check if
+        in the sub CRS if it is a compound CRS and will check if
         the source CRS is projected if it is a bound CRS.
 
         Returns
@@ -1475,7 +1470,7 @@ class CRS:
 
         This checks if the CRS is vertical.
         It will check if it has a vertical CRS
-        in the sub CRS if it is a compount CRS and will check if
+        in the sub CRS if it is a compound CRS and will check if
         the source CRS is vertical if it is a bound CRS.
 
         Returns
@@ -1761,15 +1756,15 @@ class GeographicCRS(CustomConstructorCRS):
     def __init__(
         self,
         name: str = "undefined",
-        datum: Any = "urn:ogc:def:datum:EPSG::6326",
-        ellipsoidal_cs: Any = None,
+        datum: Any = "urn:ogc:def:ensemble:EPSG::6326",
+        ellipsoidal_cs: Optional[Any] = None,
     ) -> None:
         """
         Parameters
         ----------
         name: str, default="undefined"
             Name of the CRS.
-        datum: Any, default="urn:ogc:def:datum:EPSG::6326"
+        datum: Any, default="urn:ogc:def:ensemble:EPSG::6326"
             Anything accepted by :meth:`pyproj.crs.Datum.from_user_input` or
             a :class:`pyproj.crs.datum.CustomDatum`.
         ellipsoidal_cs: Any, optional
@@ -1777,15 +1772,19 @@ class GeographicCRS(CustomConstructorCRS):
             Anything accepted by :meth:`pyproj.crs.CoordinateSystem.from_user_input`
             or an Ellipsoidal Coordinate System created from :ref:`coordinate_system`.
         """
+        datum = Datum.from_user_input(datum).to_json_dict()
         geographic_crs_json = {
             "$schema": "https://proj.org/schemas/v0.2/projjson.schema.json",
             "type": "GeographicCRS",
             "name": name,
-            "datum": Datum.from_user_input(datum).to_json_dict(),
             "coordinate_system": CoordinateSystem.from_user_input(
                 ellipsoidal_cs or Ellipsoidal2DCS()
             ).to_json_dict(),
         }
+        if datum["type"] == "DatumEnsemble":
+            geographic_crs_json["datum_ensemble"] = datum
+        else:
+            geographic_crs_json["datum"] = datum
         super().__init__(geographic_crs_json)
 
 
@@ -1806,7 +1805,7 @@ class DerivedGeographicCRS(CustomConstructorCRS):
         self,
         base_crs: Any,
         conversion: Any,
-        ellipsoidal_cs: Any = None,
+        ellipsoidal_cs: Optional[Any] = None,
         name: str = "undefined",
     ) -> None:
         """
@@ -1908,8 +1907,8 @@ class ProjectedCRS(CustomConstructorCRS):
         self,
         conversion: Any,
         name: str = "undefined",
-        cartesian_cs: Any = None,
-        geodetic_crs: Any = None,
+        cartesian_cs: Optional[Any] = None,
+        geodetic_crs: Optional[Any] = None,
     ) -> None:
         """
         Parameters
@@ -1960,7 +1959,7 @@ class VerticalCRS(CustomConstructorCRS):
         self,
         name: str,
         datum: Any,
-        vertical_cs: Any = None,
+        vertical_cs: Optional[Any] = None,
         geoid_model: Optional[str] = None,
     ) -> None:
         """
