@@ -14,13 +14,16 @@ __all__ = [
     "geodesic_version_str",
     "GeodIntermediateFlag",
     "GeodIntermediateReturn",
+    "reverse_azimuth",
 ]
 
 import math
+import warnings
 from typing import Any, Dict, List, Optional, Tuple, Union
 
 from pyproj._geod import Geod as _Geod
 from pyproj._geod import GeodIntermediateReturn, geodesic_version_str
+from pyproj._geod import reverse_azimuth as _reverse_azimuth
 from pyproj.enums import GeodIntermediateFlag
 from pyproj.exceptions import GeodError
 from pyproj.list import get_ellps_map
@@ -29,14 +32,14 @@ from pyproj.utils import DataType, _convertback, _copytobuffer
 pj_ellps = get_ellps_map()
 
 
-def _params_from_ellps_map(ellps):
+def _params_from_ellps_map(ellps: str) -> Tuple[float, float, float, float, bool]:
     """
     Build Geodesic parameters from PROJ ellips map
 
     Parameter
     ---------
     ellps: str
-        The name of the ellips in the map.
+        The name of the ellipse in the map.
 
     Returns
     -------
@@ -59,7 +62,7 @@ def _params_from_ellps_map(ellps):
     return semi_major_axis, semi_minor_axis, flattening, eccentricity_squared, sphere
 
 
-def _params_from_kwargs(kwargs):
+def _params_from_kwargs(kwargs: Dict) -> Tuple[float, float, float, float]:
     """
     Build Geodesic parameters from input kwargs:
 
@@ -75,8 +78,8 @@ def _params_from_kwargs(kwargs):
 
     Parameter
     ---------
-    ellps: str
-        The name of the ellips in the map.
+    kwargs: dict
+        The input kwargs for an ellipse.
 
     Returns
     -------
@@ -240,7 +243,14 @@ class Geod(_Geod):
         )
 
     def fwd(  # pylint: disable=invalid-name
-        self, lons: Any, lats: Any, az: Any, dist: Any, radians: bool = False
+        self,
+        lons: Any,
+        lats: Any,
+        az: Any,
+        dist: Any,
+        radians: bool = False,
+        inplace: bool = False,
+        return_back_azimuth: bool = True,
     ) -> Tuple[Any, Any, Any]:
         """
         Forward transformation
@@ -249,6 +259,9 @@ class Geod(_Geod):
         points given longitudes and latitudes of initial points,
         plus forward azimuths and distances.
 
+        .. versionadded:: 3.5.0 inplace
+        .. versionadded:: 3.5.0 return_back_azimuth
+
         Accepted numeric scalar or array:
 
         - :class:`int`
@@ -276,6 +289,13 @@ class Geod(_Geod):
         radians: bool, default=False
             If True, the input data is assumed to be in radians.
             Otherwise, the data is assumed to be in degrees.
+        inplace: bool, default=False
+            If True, will attempt to write the results to the input array
+            instead of returning a new array. This will fail if the input
+            is not an array in C order with the double data type.
+        return_back_azimuth: bool, default=True
+            If True, the third return value will be the back azimuth,
+            Otherwise, it will be the forward azimuth.
 
         Returns
         -------
@@ -284,14 +304,30 @@ class Geod(_Geod):
         scalar or array:
             Latitude(s) of terminus point(s)
         scalar or array:
-            Back azimuth(s)
+            Back azimuth(s) or Forward azimuth(s)
         """
+        try:
+            # Fast-path for scalar input, will raise if invalid types are input
+            # and we can fallback below
+            return self._fwd_point(
+                lons,
+                lats,
+                az,
+                dist,
+                radians=radians,
+                return_back_azimuth=return_back_azimuth,
+            )
+        except TypeError:
+            pass
+
         # process inputs, making copies that support buffer API.
-        inx, x_data_type = _copytobuffer(lons)
-        iny, y_data_type = _copytobuffer(lats)
-        inz, z_data_type = _copytobuffer(az)
-        ind = _copytobuffer(dist)[0]
-        self._fwd(inx, iny, inz, ind, radians=radians)
+        inx, x_data_type = _copytobuffer(lons, inplace=inplace)
+        iny, y_data_type = _copytobuffer(lats, inplace=inplace)
+        inz, z_data_type = _copytobuffer(az, inplace=inplace)
+        ind = _copytobuffer(dist, inplace=inplace)[0]
+        self._fwd(
+            inx, iny, inz, ind, radians=radians, return_back_azimuth=return_back_azimuth
+        )
         # if inputs were lists, tuples or floats, convert back.
         outx = _convertback(x_data_type, inx)
         outy = _convertback(y_data_type, iny)
@@ -305,13 +341,19 @@ class Geod(_Geod):
         lons2: Any,
         lats2: Any,
         radians: bool = False,
+        inplace: bool = False,
+        return_back_azimuth: bool = True,
     ) -> Tuple[Any, Any, Any]:
         """
+
         Inverse transformation
 
         Determine forward and back azimuths, plus distances
         between initial points and terminus points.
 
+        .. versionadded:: 3.5.0 inplace
+        .. versionadded:: 3.5.0 return_back_azimuth
+
         Accepted numeric scalar or array:
 
         - :class:`int`
@@ -338,23 +380,46 @@ class Geod(_Geod):
         radians: bool, default=False
             If True, the input data is assumed to be in radians.
             Otherwise, the data is assumed to be in degrees.
+        inplace: bool, default=False
+            If True, will attempt to write the results to the input array
+            instead of returning a new array. This will fail if the input
+            is not an array in C order with the double data type.
+        return_back_azimuth: bool, default=True
+            If True, the second return value (azi21) will be the back azimuth
+            (flipped 180 degrees), Otherwise, it will also be a forward azimuth.
 
         Returns
         -------
         scalar or array:
-            Forward azimuth(s)
+            Forward azimuth(s) (azi12)
         scalar or array:
-            Back azimuth(s)
+            Back azimuth(s) or Forward azimuth(s) (azi21)
         scalar or array:
             Distance(s) between initial and terminus point(s)
             in meters
         """
+        try:
+            # Fast-path for scalar input, will raise if invalid types are input
+            # and we can fallback below
+            return self._inv_point(
+                lons1,
+                lats1,
+                lons2,
+                lats2,
+                radians=radians,
+                return_back_azimuth=return_back_azimuth,
+            )
+        except TypeError:
+            pass
+
         # process inputs, making copies that support buffer API.
-        inx, x_data_type = _copytobuffer(lons1)
-        iny, y_data_type = _copytobuffer(lats1)
-        inz, z_data_type = _copytobuffer(lons2)
-        ind = _copytobuffer(lats2)[0]
-        self._inv(inx, iny, inz, ind, radians=radians)
+        inx, x_data_type = _copytobuffer(lons1, inplace=inplace)
+        iny, y_data_type = _copytobuffer(lats1, inplace=inplace)
+        inz, z_data_type = _copytobuffer(lons2, inplace=inplace)
+        ind = _copytobuffer(lats2, inplace=inplace)[0]
+        self._inv(
+            inx, iny, inz, ind, radians=radians, return_back_azimuth=return_back_azimuth
+        )
         # if inputs were lists, tuples or floats, convert back.
         outx = _convertback(x_data_type, inx)
         outy = _convertback(y_data_type, iny)
@@ -469,6 +534,7 @@ class Geod(_Geod):
             out_lons=None,
             out_lats=None,
             out_azis=None,
+            return_back_azimuth=False,
         )
         return list(zip(res.lons, res.lats))
 
@@ -484,12 +550,14 @@ class Geod(_Geod):
         terminus_idx: int = 1,
         radians: bool = False,
         flags: GeodIntermediateFlag = GeodIntermediateFlag.DEFAULT,
-        out_lons: Any = None,
-        out_lats: Any = None,
-        out_azis: Any = None,
+        out_lons: Optional[Any] = None,
+        out_lats: Optional[Any] = None,
+        out_azis: Optional[Any] = None,
+        return_back_azimuth: Optional[bool] = None,
     ) -> GeodIntermediateReturn:
         """
         .. versionadded:: 3.1.0
+        .. versionadded:: 3.5.0 return_back_azimuth
 
         Given a single initial point and terminus point,
         and the number of points, returns
@@ -598,12 +666,24 @@ class Geod(_Geod):
             az12(s) of the intermediate point(s)
             If None then buffers would be allocated internnaly
             unless requested otherwise by the flags
+        return_back_azimuth: bool, default=True
+            if True, out_azis will store the back azimuth,
+            Otherwise, out_azis will store the forward azimuth.
 
         Returns
         -------
         GeodIntermediateReturn:
             number of points, distance and output arrays (GeodIntermediateReturn docs)
         """
+        if return_back_azimuth is None:
+            return_back_azimuth = True
+            warnings.warn(
+                "Back azimuth is being returned by default to be compatible with fwd()"
+                "This is a breaking change for pyproj 3.5+."
+                "To avoid this warning, set return_back_azimuth=True."
+                "Otherwise, to restore old behaviour, set return_back_azimuth=False."
+                "This warning will be removed in future version."
+            )
         return super()._inv_or_fwd_intermediate(
             lon1=lon1,
             lat1=lat1,
@@ -618,6 +698,7 @@ class Geod(_Geod):
             out_lons=out_lons,
             out_lats=out_lats,
             out_azis=out_azis,
+            return_back_azimuth=return_back_azimuth,
         )
 
     def fwd_intermediate(
@@ -631,12 +712,14 @@ class Geod(_Geod):
         terminus_idx: int = 1,
         radians: bool = False,
         flags: GeodIntermediateFlag = GeodIntermediateFlag.DEFAULT,
-        out_lons: Any = None,
-        out_lats: Any = None,
-        out_azis: Any = None,
+        out_lons: Optional[Any] = None,
+        out_lats: Optional[Any] = None,
+        out_azis: Optional[Any] = None,
+        return_back_azimuth: Optional[bool] = None,
     ) -> GeodIntermediateReturn:
         """
         .. versionadded:: 3.1.0
+        .. versionadded:: 3.5.0 return_back_azimuth
 
         Given a single initial point and azimuth, number of points (npts)
         and delimiter distance between two successive points (del_s), returns
@@ -730,12 +813,24 @@ class Geod(_Geod):
             az12(s) of the intermediate point(s)
             If None then buffers would be allocated internnaly
             unless requested otherwise by the flags
+        return_back_azimuth: bool, default=True
+            if True, out_azis will store the back azimuth,
+            Otherwise, out_azis will store the forward azimuth.
 
         Returns
         -------
         GeodIntermediateReturn:
             number of points, distance and output arrays (GeodIntermediateReturn docs)
         """
+        if return_back_azimuth is None:
+            return_back_azimuth = True
+            warnings.warn(
+                "Back azimuth is being returned by default to be compatible with inv()"
+                "This is a breaking change for pyproj 3.5+."
+                "To avoid this warning, set return_back_azimuth=True."
+                "Otherwise, to restore old behaviour, set return_back_azimuth=False."
+                "This warning will be removed in future version."
+            )
         return super()._inv_or_fwd_intermediate(
             lon1=lon1,
             lat1=lat1,
@@ -750,6 +845,7 @@ class Geod(_Geod):
             out_lons=out_lons,
             out_lats=out_lats,
             out_azis=out_azis,
+            return_back_azimuth=return_back_azimuth,
         )
 
     def line_length(self, lons: Any, lats: Any, radians: bool = False) -> float:
@@ -1009,7 +1105,7 @@ class Geod(_Geod):
 
     def __repr__(self) -> str:
         # search for ellipse name
-        for (ellps, vals) in pj_ellps.items():
+        for ellps, vals in pj_ellps.items():
             if self.a == vals["a"]:
                 # self.sphere is True when self.f is zero or very close to
                 # zero (0), so prevent divide by zero.
@@ -1048,3 +1144,40 @@ class Geod(_Geod):
             return False
 
         return self.__repr__() == other.__repr__()
+
+
+def reverse_azimuth(azi: Any, radians: bool = False) -> Any:
+    """
+    Reverses the given azimuth (forward <-> backwards)
+
+    .. versionadded:: 3.5.0
+
+    Accepted numeric scalar or array:
+
+    - :class:`int`
+    - :class:`float`
+    - :class:`numpy.floating`
+    - :class:`numpy.integer`
+    - :class:`list`
+    - :class:`tuple`
+    - :class:`array.array`
+    - :class:`numpy.ndarray`
+    - :class:`xarray.DataArray`
+    - :class:`pandas.Series`
+
+    Parameters
+    ----------
+    azi: scalar or array
+        The azimuth.
+    radians: bool, default=False
+        If True, the input data is assumed to be in radians.
+        Otherwise, the data is assumed to be in degrees.
+
+    Returns
+    -------
+    scalar or array:
+        The reversed azimuth (forward <-> backwards)
+    """
+    inazi, azi_data_type = _copytobuffer(azi)
+    _reverse_azimuth(inazi, radians=radians)
+    return _convertback(azi_data_type, inazi)
